@@ -454,16 +454,24 @@ typedef int sasl_chalprompt_t(void *context, int id,
  * input:
  *  context     -- context from callback structure
  *  id          -- callback id
- *  availrealms -- available realms; NULL terminated
+ *  availrealms -- available realms; string list; NULL terminated
  * output:
  *  result      -- NUL terminated realm; NULL is equivalent to ""
  * returns SASL_OK
  */
+
+/* If there is an interaction with SASL_CB_GETREALM the challenge of
+ *  the sasl_interact_t will be of the format: {realm1, realm2,
+ *  ...}. That is a list of possible realms seperated by comma spaces
+ *  enclosed by brackets. 
+ */
+
 typedef int sasl_getrealm_t(void *context, int id,
 			    const char **availrealms,
 			    const char **result);
 #define SASL_CB_GETREALM (0x4007) /* realm to attempt authentication in */
 
+
 /* server callbacks:
  */
 /* callback to verify authorization
@@ -588,8 +596,8 @@ LIBSASL_API int sasl_idle(sasl_conn_t *conn);
  */
 typedef struct sasl_interact {
     unsigned long id;		/* same as client/user callback ID */
-    const char *challenge;
-    const char *prompt;
+    const char *challenge;      /* may be computer readable */
+    const char *prompt;         /* always human readable */
     const char *defresult;	/* default result string */
     void *result;		/* set to point to result -- this will 
 				 * be freed by the library iff it
@@ -736,11 +744,11 @@ LIBSASL_API int sasl_server_init(const sasl_callback_t *callbacks,
  *  SASL_NOMEM     -- not enough memory
  */
 LIBSASL_API int sasl_server_new(const char *service,
-		    const char *serverFQDN,
-		    const char *user_realm,
-		    const sasl_callback_t *callbacks,
-		    int secflags,
-		    sasl_conn_t **pconn);
+				const char *serverFQDN,
+				const char *user_realm,
+				const sasl_callback_t *callbacks,
+				int secflags,
+				sasl_conn_t **pconn);
 
 /* This returns a list of mechanisms in a NUL-terminated string
  *  user          -- restricts mechanisms to those available to that user
