@@ -31,6 +31,8 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include "sasl.h"
 #include "saslint.h"
 
+static int db_ok = 0;
+
 /* This provides a version of _sasl_db_getsecret and
  * _sasl_db_putsecret which work with gdbm. */
 
@@ -72,7 +74,7 @@ getsecret(void *context __attribute__((unused)),
   GDBM_FILE db;
   datum gkey, gvalue;  
 
-  if (! mechanism || ! auth_identity || ! secret)
+  if (! mechanism || ! auth_identity || ! secret || ! db_ok)
     return SASL_FAIL;
 
   result = alloc_key(mechanism,
@@ -167,3 +169,20 @@ putsecret(void *context __attribute__((unused)),
 
 sasl_server_getsecret_t *_sasl_db_getsecret = &getsecret;
 sasl_server_putsecret_t *_sasl_db_putsecret = &putsecret;
+
+int _sasl_server_check_db(const sasl_callback_t *verifyfile_cb)
+{
+    int ret;
+
+    ret = ((sasl_verifyfile_t *)(verifyfile_cb->proc))(verifyfile_cb->context,
+						       SASL_DB_PATH);
+    if (ret == SASL_OK) {
+	db_ok = 1;
+    }
+
+    if (ret == SASL_OK || ret == SASL_CONTINUE) {
+	return SASL_OK;
+    } else {
+	return ret;
+    }
+}
