@@ -163,7 +163,7 @@ _sasl_log(sasl_conn_t *conn,
 	  int errno,	/* %m */
 #else
 	  int error_value, /* %m */
-#endif //WIN32
+#endif
 	  const char *format,
 	  ...);
 
@@ -192,5 +192,12 @@ extern int _sasl_PAM_verify_password(sasl_conn_t *conn,
 				     const char *password, 
 				     const char *service,
 				     const char **reply);
+extern int _sasl_make_plain_secret(const char *salt, const 
+				   char *passwd, int passlen,
+				   sasl_secret_t **secret);
+extern int _sasl_sasldb_verify_password(sasl_conn_t *conn,
+					const char *userid, 
+					const char *passwd,
+					const char **reply);
 
 #endif /* SASLINT_H */
