@@ -1,6 +1,6 @@
 /* SASL server API implementation
  * Tim Martin
- * $Id: server.c,v 1.41 1999/07/30 19:00:19 leg Exp $
+ * $Id: server.c,v 1.46 1999/08/13 16:19:52 leg Exp $
  */
 /***********************************************************
         Copyright 1998 by Carnegie Mellon University
@@ -279,75 +279,103 @@ int sasl_setpass(sasl_conn_t *conn,
 		 int flags,
 		 const char **errstr)
 {
-  int result=SASL_OK, tmpresult;
-  sasl_server_conn_t *s_conn=  (sasl_server_conn_t *) conn;
-  mechanism_t *m;
+    int result=SASL_OK, tmpresult;
+    sasl_server_conn_t *s_conn = (sasl_server_conn_t *) conn;
+    mechanism_t *m;
+    sasl_getopt_t *getopt;
+    void *context;
+    const char *plainmech;
 
-  /* Zowie -- we have the user's plaintext password.
-   * Let's tell all our mechanisms about it...
-   */
+    /* Zowie -- we have the user's plaintext password.
+     * Let's tell all our mechanisms about it...
+     */
 
-  if (! conn || ! pass)
-    return SASL_FAIL;
+    if (!conn || !pass || !mechlist)
+	return SASL_FAIL;
 
-  if (! mechlist)		/* if haven't init'ed yet */
-    return SASL_FAIL;
+    VL(("Setting password for \"%s\" to \"%*s\" (len is %d)\n",
+	user, passlen, pass, passlen));
 
-  VL(("Setting password for \"%s\" to \"%*s\" (len is %d)\n",
-      user, passlen, pass, passlen));
+    _sasl_log(conn, SASL_LOG_INFO, NULL, 0, 0,
+	      "updating secrets for %s", user);
 
-  /* copy info into sparams */
-  s_conn->sparams->serverFQDN=conn->serverFQDN;
-  s_conn->sparams->service=conn->service;
-  s_conn->sparams->user_realm=s_conn->user_realm;
-
-  _sasl_log(conn,
-	    SASL_LOG_WARNING,
-	    NULL,
-	    0, 0,
-	    "Updating secrets for %s",
-	    user);
-
-  for (m = mechlist->mech_list;
-       m;
-       m = m->next)
-    if (m->plug->setpass)
-    {
-      VL(("Setting it for mech %s\n", m->plug->mech_name));
-      tmpresult=m->plug->setpass(m->plug->glob_context,
-			   ((sasl_server_conn_t *)conn)->sparams,
-			   user,
-			   pass,
-			   passlen,
-			   flags,
-			   errstr);
-      if (tmpresult!=SASL_OK)
-      {
-	VL(("%s returned %i\n",m->plug->mech_name, tmpresult));
-	result = SASL_FAIL;
-	_sasl_log(conn,
-		  SASL_LOG_WARNING,
-		  m->plug->mech_name,
-		  tmpresult,
-#ifndef WIN32
-		  errno,
-#else
-		  GetLastError(),
-#endif //WIN32
-		  "Failed to set secret for %s: %z",
-		  user);
-      } else {
-	VL(("%s succeeded!\n",m->plug->mech_name));
-	_sasl_log(conn,
-		 SASL_LOG_WARNING,
-		  m->plug->mech_name,
-		  0, 0,
-		  "Set secret for %s",
-		  user);
-      }
+    if (_sasl_getcallback(conn, SASL_CB_GETOPT, 
+			  &getopt, &context) == SASL_OK) {
+	getopt(context, NULL, "pwcheck_method", &plainmech, NULL);
     }
 
-  return result;
+    if (plainmech != NULL && !strcmp(plainmech, "sasldb")) {
+	/* we store the user's password for plaintext checking */
+	sasl_secret_t *sec = NULL;
+	char salt[16];
+	sasl_rand_t *rpool;
+	int ret;
+	sasl_server_putsecret_t *putsec;
+	void *context;
+
+	ret = sasl_randcreate(&rpool);
+	if (ret == SASL_OK) {
+	    sasl_rand(rpool, salt, 16);
+	    ret = _sasl_make_plain_secret(salt, pass, passlen, &sec);
+	}
+	if (ret == SASL_OK) {
+	    ret = _sasl_getcallback(conn, SASL_CB_SERVER_PUTSECRET, &putsec,
+				    &context);
+	}
+	if (ret == SASL_OK) {
+	    ret = putsec(context, "PLAIN", user, sec);
+	}
+	if (ret != SASL_OK) {
+	    _sasl_log(conn, SASL_LOG_ERR, NULL, ret, 0,
+		      "failed to set plaintext secret for %s: %z", user);
+	}
+	if (rpool) {
+	    sasl_randfree(&rpool);
+	}
+	if (sec) {
+	    sasl_free_secret(&sec);
+	}
+    }
+
+    /* copy info into sparams */
+    s_conn->sparams->serverFQDN = conn->serverFQDN;
+    s_conn->sparams->service = conn->service;
+    s_conn->sparams->user_realm = s_conn->user_realm;
+
+    /* now we let the mechanisms set their secrets */
+    for (m = mechlist->mech_list; m; m = m->next) {
+	if (m->plug->setpass) {
+	    VL(("Setting it for mech %s\n", m->plug->mech_name));
+	    tmpresult = m->plug->setpass(m->plug->glob_context,
+					 ((sasl_server_conn_t *)conn)->sparams,
+					 user,
+					 pass,
+					 passlen,
+					 flags,
+					 errstr);
+	    if (tmpresult != SASL_OK) {
+		VL(("%s returned %i\n",m->plug->mech_name, tmpresult));
+		result = SASL_FAIL;
+		_sasl_log(conn, SASL_LOG_ERR, m->plug->mech_name, tmpresult,
+#ifndef WIN32
+			  errno,
+#else
+			  GetLastError(),
+#endif
+			  "failed to set secret for %s: %z", user);
+	    } else {
+		VL(("%s succeeded!\n",m->plug->mech_name));
+		_sasl_log(conn,
+			  SASL_LOG_INFO,
+			  m->plug->mech_name,
+			  0, 0,
+			  "set secret for %s",
+			  user);
+	    }
+	}
+    }
+
+    return result;
 }
 
 
@@ -526,11 +554,8 @@ static int load_config(const sasl_callback_t *verifyfile_cb)
   config_filename = sasl_ALLOC(len);
   if (! config_filename) return SASL_NOMEM; 
 
-  strcpy(config_filename, path_to_config);
-  strcat(config_filename,"/");
-  strcat(config_filename, global_callbacks.appname);
-  strcat(config_filename, ".conf");
-
+  snprintf(config_filename, len, "%s/%s.conf", path_to_config, 
+	   global_callbacks.appname);
 
   /* Ask the application if it's safe to use this file */
   result = ((sasl_verifyfile_t *)(verifyfile_cb->proc))(verifyfile_cb->context,
@@ -545,7 +570,9 @@ static int load_config(const sasl_callback_t *verifyfile_cb)
     result=sasl_config_init(config_filename);
 
   sasl_FREE(config_filename);
-  sasl_FREE(path_to_config);
+  if (*path_to_config) { /* was path_to_config allocated? */
+      sasl_FREE(path_to_config);
+  }
 
   return result;
 }
@@ -960,23 +987,31 @@ int sasl_listmech(sasl_conn_t *conn,
   int lup;
   mechanism_t *listptr;  
   int resultlen;
+  int flag;
+  const char *mysep;
 
   if (! conn || ! result)
     return SASL_FAIL;
 
-  if (plen!=NULL)
-    *plen=0;
-  if (pcount!=NULL)
-    *pcount=0;
+  if (plen != NULL)
+      *plen = 0;
+  if (pcount != NULL)
+      *pcount = 0;
+
+  if (sep) {
+      mysep = sep;
+  } else {
+      mysep = " ";
+  }
 
   if (! mechlist)
     return SASL_FAIL;
 
-  if (mechlist->mech_length<=0)
+  if (mechlist->mech_length <= 0)
     return SASL_NOMECH;
 
   resultlen = (prefix ? strlen(prefix) : 0)
-            + (sep ? strlen(sep) : 1) * (mechlist->mech_length - 1)
+            + (strlen(mysep) * (mechlist->mech_length - 1))
 	    + mech_names_len()
             + (suffix ? strlen(suffix) : 0)
 	    + 1;
@@ -988,40 +1023,35 @@ int sasl_listmech(sasl_conn_t *conn,
   else
     **result = '\0';
 
-  listptr=mechlist->mech_list;  
+  listptr = mechlist->mech_list;  
    
+  flag = 0;
   /* make list */
-  for (lup=0;lup<mechlist->mech_length;lup++)
-  {
-    /* if user has rights add to list */
-    /* XXX This should be done with a callback function */
-    if (mech_permitted(conn, listptr->plug))
-    {
-      if (pcount!=NULL)
-	(*pcount)++;
+  for (lup = 0; lup < mechlist->mech_length; lup++) {
+      /* currently, we don't use the "user" parameter for anything */
+      if (mech_permitted(conn, listptr->plug)) {
+	  if (pcount != NULL)
+	      (*pcount)++;
 
-      /* print seperator */      
-      if (lup>0)
-      {
-	if (sep)
-	  strcat(*result,sep);
-	else
-	  strcat(*result," "); /* if seperator is NULL give it space */
+	  /* print seperator */
+	  if (flag) {
+	      strcat(*result, mysep);
+	  } else {
+	      flag = 1;
+	  }
+
+	  /* now print the mechanism name */
+	  strcat(*result, listptr->plug->mech_name);
       }
 
-      /* now print the mechanism name */
-      strcat(*result,listptr->plug->mech_name);
-
-    }
-
-    listptr=listptr->next;
+      listptr = listptr->next;
   }
 
   if (suffix)
-    strcat(*result,suffix);
+      strcat(*result,suffix);
 
   if (plen!=NULL)
-    *plen=strlen(*result);
+      *plen=strlen(*result);
 
   return SASL_OK;
   
@@ -1037,25 +1067,30 @@ static int _sasl_checkpass(sasl_conn_t *conn,
 #ifdef HAVE_PAM
     if (!strcmp(mech, "PAM")) {
 	result = _sasl_PAM_verify_password(conn, user, pass, service, NULL);
-    }
+    } else
 #endif
 #ifndef SASL_MINIMAL_SERVER
     if (!strcmp(mech, "passwd")) {
 	result = _sasl_passwd_verify_password(conn, user, pass, NULL);
-    }
+    } else
 #endif //SASL_MINIMAL_SERVER
     if (!strcmp(mech, "shadow")) {
 	result = _sasl_shadow_verify_password(conn, user, pass, NULL);
-    }
+    } else
 
 #ifdef HAVE_KRB
     if (!strcmp(mech, "kerberos_v4")) {
 	/* check against krb */
 	result = _sasl_kerberos_verify_password(conn, user, pass, 
 						service, NULL);
-    }
+    } else
 #endif
 
+    if (!strcmp(mech, "sasldb")) {
+	/* check sasl database */
+	result = _sasl_sasldb_verify_password(conn, user, pass, NULL);
+    }
+
     return result;
 }
 
