@@ -26,11 +26,15 @@ SOFTWARE.
 
 #include <config.h>
 #include <gssapi.h>
+#ifdef WIN32
+#include <winsock.h>
+#else
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
+#endif /*WIN32*/
 #include <fcntl.h>
 #include <stdio.h>
 #include <sasl.h>
@@ -246,7 +250,7 @@ sasl_gss_decode(void *context, const char *input, unsigned inputlen,
   
     if (inputlen != 0) { /* we received more then just one packet */
 	char *extra = NULL;
-	int extralen;
+	unsigned extralen;
 
 	sasl_gss_decode(text, input, inputlen, &extra, &extralen);
 	if (extra != NULL) {
