@@ -1,5 +1,6 @@
 /* Kerberos4 SASL plugin
  * Tim Martin 
+ * $Id: kerberos4.c,v 1.38 1999/08/21 02:37:26 leg Exp $
  */
 /***********************************************************
         Copyright 1998 by Carnegie Mellon University
@@ -85,7 +86,8 @@ extern int gethostname(char *, int);
 typedef struct context {
   int state;
 
-  unsigned long challenge;         /* this is the challenge (32 bit int) used for the authentication */
+  int challenge;         /* this is the challenge (32 bit int) used 
+			    for the authentication */
 
   char *service;                   /* kerberos service */
   char instance[ANAME_SZ];
@@ -123,6 +125,9 @@ typedef struct context {
   int needsize;
   int secflags; /* client/server supports layers? */
 
+  long time_sec; /* These are used to make sure we are getting */
+  char time_5ms; /* strictly increasing timestamps */
+
 } context_t;
 
 static char *srvtab = NULL;
@@ -229,10 +234,23 @@ static int privacy_decode(void *context,
     len= krb_rd_priv((char *) text->buffer,text->size,  text->init_keysched, 
 		     text->session, text->ip_remote, text->ip_local, data);
 
+    /* see if the krb library gave us a failure */
     if (len != 0) {
-	return SASL_FAIL - (len * 100);
+	return SASL_FAIL;
     }
 
+    /* check to make sure the timestamps are ok */
+    if ((data->time_sec < text->time_sec) || /* if an earlier time */
+	(((data->time_sec == text->time_sec) && /* or the exact same time */
+	 (data->time_5ms < text->time_5ms)))) 
+    {
+      return SASL_FAIL;
+    }
+    text->time_sec = data->time_sec;
+    text->time_5ms = data->time_5ms;
+
+
+
     *output = text->malloc(data->app_length + 1);
     if ((*output) == NULL) {
 	return SASL_NOMEM;
@@ -370,12 +388,23 @@ static int integrity_decode(void *context, const char *input, unsigned inputlen,
 		     (text->ip_remote), (text->ip_local), data);
 
 
-
+    /* see if the krb library found a problem with what we were given */
     if (len!=0)
     {
-      return SASL_FAIL-(len*100);
+      return SASL_FAIL;
     }
 
+    /* check to make sure the timestamps are ok */
+    if ((data->time_sec < text->time_sec) || /* if an earlier time */
+	(((data->time_sec == text->time_sec) && /* or the exact same time */
+	 (data->time_5ms < text->time_5ms)))) 
+    {
+      return SASL_FAIL;
+    }
+    text->time_sec = data->time_sec;
+    text->time_5ms = data->time_5ms;
+
+
     *output=text->malloc(data->app_length+1);
     if ((*output) == NULL) return SASL_NOMEM;
  
@@ -417,6 +446,9 @@ new_text(sasl_client_params_t *p, context_t **text)
     ret->buffer = NULL;
     ret->bufsize = 0;
 
+    ret->time_sec=0;
+    ret->time_5ms=0;
+
     ret->state = 0;  
     *text = ret;
 
@@ -431,11 +463,8 @@ server_start(void *glob_context __attribute__((unused)),
 	     void **conn,
 	     const char **errstr)
 {
-#ifndef WIN32
-  context_t *text;
-#endif //win32
   if (errstr)
-    *errstr = NULL;
+      *errstr = NULL;
 
   return new_text(sparams, (context_t **) conn);
 }
@@ -501,10 +530,10 @@ static int server_continue_step (void *conn_context,
 
   if (text->state==0)
   {    
-    /* random 32-bit number */
-    unsigned long randocts,nchal;
+      /* random 32-bit number */
+      int randocts, nchal;
 
-    /* shouldn't we check for erroneous client input here?!? */
+      /* shouldn't we check for erroneous client input here?!? */
 
     VL(("KERBEROS_V4 Step 1\n"));
 
@@ -526,7 +555,7 @@ static int server_continue_step (void *conn_context,
 
   if (text->state==1)
   {
-    unsigned long nchal;
+    int nchal;
     unsigned char sout[8];  
     AUTH_DAT ad;
     KTEXT_ST ticket;
@@ -542,7 +571,7 @@ static int server_continue_step (void *conn_context,
     for (lup=0;lup<clientinlen;lup++)      
       ticket.dat[lup]=clientin[lup];
 
-    text->realm=krb_realmofhost(sparams->serverFQDN);
+    text->realm = krb_realmofhost(sparams->serverFQDN);
 
     /* get instance */
     strncpy (text->instance, krb_get_phost (sparams->serverFQDN),
@@ -589,7 +618,7 @@ static int server_continue_step (void *conn_context,
       sout[4] |= KRB_SECFLAG_CREDENTIALS;
     sout[5]=0x00;  /* max ciphertext buffer size */
     sout[6]=0xFF;  /* let's say we can support up to 64K */
-    sout[7]=0xFF;  /* no inherint inability with our layers to support more */
+    sout[7]=0xFF;  /* no inherent inability with our layers to support more */
 
     memcpy(text->session, ad.session, 8);
     memcpy(text->pname, ad.pname, sizeof(text->pname));
@@ -618,8 +647,8 @@ static int server_continue_step (void *conn_context,
   if (text->state==2)
   {
     int result;
-    unsigned long testnum;
-    int lup;
+    int testnum;
+    int lup, flag;
     unsigned char in[1024];
 
     for (lup=0;lup<clientinlen;lup++)
@@ -689,8 +718,14 @@ static int server_continue_step (void *conn_context,
     
     {
       size_t len = strlen(text->pname);
-      if (text->pinst[0])
+      if (text->pinst[0]) {
 	len += strlen(text->pinst) + 1 /* for the . */;
+      }
+      flag = 0;
+      if (strcmp(text->realm, text->prealm)) {
+	len += strlen(text->prealm) + 1 /* for the @ */;
+	flag = 1;
+      }
 
       oparams->authid = sparams->utils->malloc(len + 1);
       if (! oparams->authid)
@@ -700,6 +735,10 @@ static int server_continue_step (void *conn_context,
 	strcat(oparams->authid, ".");
 	strcat(oparams->authid, text->pinst);
       }
+      if (flag) {
+	strcat(oparams->authid, "@");
+	strcat(oparams->authid, text->prealm);
+      }  
 
       if (in[8]) {
 	  oparams->user = sparams->utils->malloc(strlen(in + 8) + 1);
@@ -891,8 +930,8 @@ static int client_continue_step (void *conn_context,
   /* challenge #2 */
   if (text->state==2)
   {
-    unsigned long testnum;
-    unsigned long nchal;    
+    int testnum;
+    int nchal;    
     unsigned char sout[1024];
     unsigned len;
     unsigned char in[8];
@@ -1147,7 +1186,7 @@ static int client_continue_step (void *conn_context,
 }
 
 static const long client_required_prompts[] = {
-  SASL_CB_AUTHNAME,
+  SASL_CB_USER,
   SASL_CB_LIST_END
 };
 
