@@ -248,6 +248,7 @@ samp_send(const char *buffer,
   buf = malloc(alloclen);
   if (! buf)
     osfail();
+
   result = sasl_encode64(buffer, length, buf, alloclen, &len);
   if (result != SASL_OK)
     saslfail(result, "Encoding data in base64", NULL);
@@ -261,9 +262,12 @@ samp_recv()
   unsigned len;
   int result;
   
-  if (! fgets(buf, SAMPLE_SEC_BUF_SIZE, stdin)
-      || strncmp(buf, "C: ", 3))
+  if (! fgets(buf, SAMPLE_SEC_BUF_SIZE, stdin))
     fail("Unable to parse input");
+
+  if (strncmp(buf, "C: ", 3)!=0)
+    fail("Line must start with 'C: '");
+    
   result = sasl_decode64(buf + 3, strlen(buf + 3), buf, &len);
   if (result != SASL_OK)
     saslfail(result, "Decoding data from base64", NULL);
@@ -271,6 +275,7 @@ samp_recv()
   return len;
 }
 
+
 int
 main(int argc, char *argv[])
 {
@@ -538,7 +543,10 @@ main(int argc, char *argv[])
     if (data) {
       puts("Sending response...");
       samp_send(data, len);
+#ifndef WIN32
+	  /*win32 crashes on this free*/
       free(data);
+#endif /*win32*/
     } else
       fail("No data to send--something's wrong");
     puts("Waiting for client reply...");
@@ -571,5 +579,6 @@ main(int argc, char *argv[])
   else
     printf("SSF: %d\n", *ssf);
 
+  printf("a\n");
   return (EXIT_SUCCESS);
 }
