@@ -33,7 +33,7 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #define PACKAGE "cyrus-sasl"
 
 /* Our version */
-#define VERSION "1.5.3"
+#define VERSION "1.5.5"
 
 /* Registry key that contains the locations of the plugins */
 #define SASL_KEY "SOFTWARE\\Carnegie Mellon\\Project Cyrus\\SASL Library\\Available Plugins"
@@ -71,8 +71,8 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #define L_DEFAULT_GUARD (0)
 #define I_DEFAULT_GUARD (0)
 #define I(foo)
-#define VL(foo)
-// #define VL(foo)  printf foo;
+//#define VL(foo)
+#define VL(foo)  printf foo;
 #define VLP(foo,bar)
 
 #define __attribute__(foo)
